"""Dataclass definitions for the Catppuccin palette data structure."""
from dataclasses import dataclass
from typing import Iterator


@dataclass(frozen=True)
class RGB:
    """Color represented as red, green, and blue (all 0-255)."""

    r: int
    g: int
    b: int


@dataclass(frozen=True)
class HSL:
    """Color represented as hue (0-359), saturation (0-1), and lightness (0-1)."""

    h: float
    s: float
    l: float  # noqa: E741


@dataclass(frozen=True)
class Color:
    """A single color in the Catppuccin palette."""

    name: str
    order: int
    hex: str
    rgb: RGB
    hsl: HSL
    accent: bool


@dataclass(frozen=True)
class FlavorColors:
    """All of the colors for a particular flavor of Catppuccin."""

    rosewater: Color
    flamingo: Color
    pink: Color
    mauve: Color
    red: Color
    maroon: Color
    peach: Color
    yellow: Color
    green: Color
    teal: Color
    sky: Color
    sapphire: Color
    blue: Color
    lavender: Color
    text: Color
    subtext1: Color
    subtext0: Color
    overlay2: Color
    overlay1: Color
    overlay0: Color
    surface2: Color
    surface1: Color
    surface0: Color
    base: Color
    mantle: Color
    crust: Color

    def __iter__(self) -> Iterator[Color]:
        """Iterate over colors in the flavor."""
        yield from [
            self.rosewater,
            self.flamingo,
            self.pink,
            self.mauve,
            self.red,
            self.maroon,
            self.peach,
            self.yellow,
            self.green,
            self.teal,
            self.sky,
            self.sapphire,
            self.blue,
            self.lavender,
            self.text,
            self.subtext1,
            self.subtext0,
            self.overlay2,
            self.overlay1,
            self.overlay0,
            self.surface2,
            self.surface1,
            self.surface0,
            self.base,
            self.mantle,
            self.crust,
        ]


@dataclass(frozen=True)
class Flavor:
    """A flavor is a collection of colors.

    Catppuccin has four flavors; Latte, Frappé, Macchiato, and Mocha.
    Can be iterated over, in which case the colors are yielded in order.
    """

    name: str
    order: int
    dark: bool
    colors: FlavorColors


@dataclass(frozen=True)
class Palette:
    """The top-level type that encompasses the Catppuccin palette data structure.

    Primarily used via the PALETTE constant.
    Can be iterated over, in which case the flavors are yielded in the canonical
    order: Latte, Frappé, Macchiato, Mocha.
    """

    latte: Flavor
    frappe: Flavor
    macchiato: Flavor
    mocha: Flavor

    def __iter__(self) -> Iterator[Flavor]:
        """Iterate over flavors in the palette."""
        yield from [self.latte, self.frappe, self.macchiato, self.mocha]



"""Rich themes for all Catppuccin flavors."""
from rich.theme import Theme

from catppuccin import PALETTE
from catppuccin.models import FlavorColors


def _make_theme(colors: FlavorColors) -> Theme:
    return Theme(
        {
            "rosewater": colors.rosewater.hex,
            "flamingo": colors.flamingo.hex,
            "pink": colors.pink.hex,
            "mauve": colors.mauve.hex,
            "red": colors.red.hex,
            "maroon": colors.maroon.hex,
            "peach": colors.peach.hex,
            "yellow": colors.yellow.hex,
            "green": colors.green.hex,
            "teal": colors.teal.hex,
            "sky": colors.sky.hex,
            "sapphire": colors.sapphire.hex,
            "blue": colors.blue.hex,
            "lavender": colors.lavender.hex,
            "text": colors.text.hex,
            "subtext1": colors.subtext1.hex,
            "subtext0": colors.subtext0.hex,
            "overlay2": colors.overlay2.hex,
            "overlay1": colors.overlay1.hex,
            "overlay0": colors.overlay0.hex,
            "surface2": colors.surface2.hex,
            "surface1": colors.surface1.hex,
            "surface0": colors.surface0.hex,
            "base": colors.base.hex,
            "mantle": colors.mantle.hex,
            "crust": colors.crust.hex,
        },
    )


latte = _make_theme(PALETTE.latte.colors)
frappe = _make_theme(PALETTE.frappe.colors)
macchiato = _make_theme(PALETTE.macchiato.colors)
mocha = _make_theme(PALETTE.mocha.colors)



# ruff: noqa: ERA001
"""Pygments styles for all Catppuccin flavors."""
from __future__ import annotations

from typing import TYPE_CHECKING

from pygments.style import Style
from pygments.token import (
    Comment,
    Error,
    Generic,
    Keyword,
    Literal,
    Name,
    Number,
    Operator,
    Other,
    Punctuation,
    String,
    Text,
    Whitespace,
    _TokenType,
)

from catppuccin import PALETTE

if TYPE_CHECKING:
    from catppuccin.models import FlavorColors


def _make_styles(colors: FlavorColors) -> dict[_TokenType, str]:
    return {
        Comment: colors.overlay0.hex,
        Comment.Hashbang: colors.overlay0.hex,
        Comment.Multiline: colors.overlay0.hex,
        Comment.Preproc: colors.pink.hex,
        Comment.Single: colors.overlay0.hex,
        Comment.Special: colors.overlay0.hex,
        Generic: colors.text.hex,
        Generic.Deleted: colors.red.hex,
        Generic.Emph: f"{colors.text.hex} underline",
        Generic.Error: colors.text.hex,
        Generic.Heading: f"{colors.text.hex} bold",
        Generic.Inserted: f"{colors.text.hex} bold",
        Generic.Output: colors.overlay0.hex,
        Generic.Prompt: colors.text.hex,
        Generic.Strong: colors.text.hex,
        Generic.Subheading: f"{colors.text.hex} bold",
        Generic.Traceback: colors.text.hex,
        Error: colors.text.hex,
        # `as`
        Keyword: colors.mauve.hex,
        Keyword.Constant: colors.pink.hex,
        Keyword.Declaration: f"{colors.teal.hex} italic",
        # `from`, `import`
        Keyword.Namespace: colors.mauve.hex,
        Keyword.Pseudo: colors.pink.hex,
        Keyword.Reserved: colors.pink.hex,
        Keyword.Type: colors.teal.hex,
        Literal: colors.text.hex,
        Literal.Date: colors.text.hex,
        # from xxx import NAME
        # NAME = NAME
        # NAME.NAME()
        Name: colors.text.hex,
        Name.Attribute: colors.green.hex,
        # `len`, `print`
        Name.Builtin: f"{colors.peach.hex} italic",
        # `self`
        Name.Builtin.Pseudo: colors.red.hex,
        # class Name.Class:
        Name.Class: colors.yellow.hex,
        Name.Constant: colors.text.hex,
        Name.Decorator: colors.text.hex,
        Name.Entity: colors.text.hex,
        Name.Exception: colors.text.hex,
        # def __Name.Label__(
        Name.Function: colors.sapphire.hex,
        Name.Label: f"{colors.teal.hex} italic",
        Name.Namespace: colors.text.hex,
        Name.Other: colors.text.hex,
        Name.Tag: colors.pink.hex,
        Name.Variable: f"{colors.text.hex} italic",
        Name.Variable.Class: f"{colors.yellow.hex} italic",
        Name.Variable.Global: f"{colors.text.hex} italic",
        Name.Variable.Instance: f"{colors.text.hex} italic",
        Number: colors.peach.hex,
        Number.Bin: colors.peach.hex,
        Number.Float: colors.peach.hex,
        Number.Hex: colors.peach.hex,
        Number.Integer: colors.peach.hex,
        Number.Integer.Long: colors.peach.hex,
        Number.Oct: colors.peach.hex,
        # `=`
        Operator: colors.sky.hex,
        # `not`, `in`
        Operator.Word: colors.mauve.hex,
        Other: colors.text.hex,
        # `(`, `)`, `,`, `[`, `]`, `:`
        Punctuation: colors.overlay2.hex,
        String: colors.green.hex,
        String.Backtick: colors.green.hex,
        String.Char: colors.green.hex,
        String.Doc: colors.green.hex,
        String.Double: colors.green.hex,
        String.Escape: colors.green.hex,
        String.Heredoc: colors.green.hex,
        String.Interpol: colors.green.hex,
        String.Other: colors.green.hex,
        String.Regex: colors.green.hex,
        String.Single: colors.green.hex,
        String.Symbol: colors.green.hex,
        Text: colors.text.hex,
        Whitespace: colors.text.hex,
    }


class LatteStyle(Style):
    """Catppuccin Latte pygments style."""

    _colors = PALETTE.latte.colors

    background_color = _colors.base.hex
    line_number_background_color = _colors.mantle.hex
    line_number_color = _colors.text.hex

    styles = _make_styles(_colors)


class FrappeStyle(Style):
    """Catppuccin Frappé pygments style."""

    _colors = PALETTE.frappe.colors

    background_color = _colors.base.hex
    line_number_background_color = _colors.mantle.hex
    line_number_color = _colors.text.hex

    styles = _make_styles(_colors)


class MacchiatoStyle(Style):
    """Catppuccin Macchiato pygments style."""

    _colors = PALETTE.macchiato.colors

    background_color = _colors.base.hex
    line_number_background_color = _colors.mantle.hex
    line_number_color = _colors.text.hex

    styles = _make_styles(_colors)


class MochaStyle(Style):
    """Catppuccin Mocha pygments style."""

    _colors = PALETTE.mocha.colors

    background_color = _colors.base.hex
    line_number_background_color = _colors.mantle.hex
    line_number_color = _colors.text.hex

    styles = _make_styles(_colors)
